#ifndef LIBWIND_H
#define LIBWIND_H

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

typedef struct wind_client_t
{
    int fd;
    int operation_in_progress;
    uint8_t saved_id;
    int(*handler)(struct wind_client_t*, uint8_t, void*);
    void* arg;
} wind_client_t;

struct wind_point
{
    int x;
    int y;
};

struct wind_rect
{
    struct wind_point pos;
    int width;
    int height;
};

typedef struct {
    int id;
    void* canvas;
    struct wind_rect rect;
} wind_window_t;

enum wind_server_message
{
    wind_ClientError,
    wind_CreateWindow,
    wind_RemoveShared,
    wind_SetWindowTitle,
    wind_Invalidate,
    wind_CloseWindow,
    wind_GetScreenRect,
    wind_SetTitlebarHeight,
    wind_SetSpecialWindowAttributes,
};

enum wind_client_message
{
    wind_ServerError,
    wind_CreateWindowResponse,
    wind_WindowCloseRequest,
    wind_MouseEvent,
    wind_MouseLeave,
    wind_GetScreenRectResponse,
    wind_KeyEvent,
};

#define WIND_STRING_LENGTH 256

struct wind_create_window_request
{
    struct wind_rect rect;
};

struct wind_remove_shared_request
{
    int window;
};

struct wind_set_window_title_request
{
    int window;
    char title[WIND_STRING_LENGTH];
};

struct wind_invalidate_request
{
    int window;
};

struct wind_close_window_request
{
    int window;
};

struct wind_get_screen_rect_request
{
    int _padding;
};

struct wind_set_titlebar_height_request
{
    int window;
    int height;
};

enum wind_window_attributes
{
    wind_UNFOCUSEABLE = 1,
};

struct wind_set_special_window_attributes_request
{
    int window;
    enum wind_window_attributes attributes;
};

struct wind_create_window_response
{
    int window;
    char shm_path[WIND_STRING_LENGTH];
};

struct wind_window_close_request
{
    int window;
};

enum wind_mouse_button
{
    wind_mouse_Left = (1 << 0),
    wind_mouse_Middle = (1 << 1),
    wind_mouse_Right = (1 << 2),
};

struct wind_mouse_event_request
{
    int window;
    struct wind_point position;
    enum wind_mouse_button buttons;
};

struct wind_mouse_leave_request
{
    int window;
};

struct wind_get_screen_rect_response
{
    struct wind_rect rect;
};

enum wind_keycode
{
    // Function keys
    wind_key_F1,
    wind_key_F2,
    wind_key_F3,
    wind_key_F4,
    wind_key_F5,
    wind_key_F6,
    wind_key_F7,
    wind_key_F8,
    wind_key_F9,
    wind_key_F10,
    wind_key_F11,
    wind_key_F12,
    // System keys
    wind_key_Esc,
    wind_key_PrtScr,
    wind_key_Pause,
    wind_key_Super,
    wind_key_Menu,
    // Modifier keys
    wind_key_LeftShift,
    wind_key_RightShift,
    wind_key_LeftAlt,
    wind_key_RightAlt, // or AltGr on some keyboards
    wind_key_LeftControl,
    wind_key_RightControl,
    // Navigation keys
    wind_key_Tab,
    wind_key_Home,
    wind_key_End,
    wind_key_PageUp,
    wind_key_PageDown,
    wind_key_RightArrow,
    wind_key_LeftArrow,
    wind_key_UpArrow,
    wind_key_DownArrow,
    // Editing keys
    wind_key_Backspace,
    wind_key_Enter,
    wind_key_Insert,
    wind_key_Delete,
    wind_key_KeypadEnter,
    // Lock keys
    wind_key_ScrollLock,
    wind_key_CapsLock,
    wind_key_NumLock,
    // Keypad keys
    wind_key_Keypad0,
    wind_key_Keypad1,
    wind_key_Keypad2,
    wind_key_Keypad3,
    wind_key_Keypad4,
    wind_key_Keypad5,
    wind_key_Keypad6,
    wind_key_Keypad7,
    wind_key_Keypad8,
    wind_key_Keypad9,
    wind_key_KeypadDot,
    wind_key_KeypadPlus,
    wind_key_KeypadMinus,
    wind_key_KeypadMul,
    wind_key_KeypadDiv,
    // Character keys (depending on keyboard layout), examples in US QWERTY
    wind_key_CH00, // `
    wind_key_CH01, // 1
    wind_key_CH02, // 2
    wind_key_CH03, // 3
    wind_key_CH04, // 4
    wind_key_CH05, // 5
    wind_key_CH06, // 6
    wind_key_CH07, // 7
    wind_key_CH08, // 8
    wind_key_CH09, // 9
    wind_key_CH10, // 0
    wind_key_CH11, // -
    wind_key_CH12, // =
    wind_key_CH13, // Q
    wind_key_CH14, // W
    wind_key_CH15, // E
    wind_key_CH16, // R
    wind_key_CH17, // T
    wind_key_CH18, // Y
    wind_key_CH19, // U
    wind_key_CH20, // I
    wind_key_CH21, // O
    wind_key_CH22, // P
    wind_key_CH23, // [
    wind_key_CH24, // ]
    wind_key_CH25, // A
    wind_key_CH26, // S
    wind_key_CH27, // D
    wind_key_CH28, // F
    wind_key_CH29, // G
    wind_key_CH30, // H
    wind_key_CH31, // J
    wind_key_CH32, // K
    wind_key_CH33, // L
    wind_key_CH34, // ;
    wind_key_CH35, // '
    wind_key_CH36, // #
    wind_key_CH37, // Backslash
    wind_key_CH38, // Z
    wind_key_CH39, // X
    wind_key_CH40, // C
    wind_key_CH41, // V
    wind_key_CH42, // B
    wind_key_CH43, // N
    wind_key_CH44, // M
    wind_key_CH45, // ,
    wind_key_CH46, // .
    wind_key_CH47, // /
    wind_key_CH48, // Space
    // Multimedia keys
    wind_key_MediaPrev,
    wind_key_MediaNext,
    wind_key_MediaMute,
    wind_key_MediaCalc,
    wind_key_MediaPlay,
    wind_key_MediaStop,
    wind_key_MediaVolDown,
    wind_key_MediaVolUp,
    // WWW keys
    wind_key_WWWHome,
    wind_key_WWWSearch,
    wind_key_WWWFavorites,
    wind_key_WWWRefresh,
    wind_key_WWWStop,
    wind_key_WWWForward,
    wind_key_WWWBack,
    wind_key_WWWMyComputer,
    wind_key_WWWEmail,
    wind_key_WWWSelect,
    // Power keys
    wind_key_ACPIPower,
    wind_key_ACPISleep,
    wind_key_ACPIWake,
    // Unknown key
    wind_key_Unknown,
};

struct wind_key_event_request
{
    int window;

    bool pressed;

    char letter;
    char key;
    uint8_t code;
    int modifiers;
};

#define WIND_SOCKET_PATH "/tmp/wind.sock"

int wind_connect(wind_client_t* client, const char* path, int blocking);
int wind_send_request(wind_client_t* client, uint8_t request_id, const void* request, size_t request_size);
int wind_read_response(wind_client_t* client, uint8_t response_id, void* response, size_t response_size);
int wind_wait_for_sync_reply(wind_client_t* client, uint8_t response_id, void* response, size_t response_size);
int wind_client_fd(wind_client_t* client);
int wind_disconnect(wind_client_t* client);

void wind_set_handler(wind_client_t* client, int(*handler)(wind_client_t*, uint8_t, void*), void* arg);

int wind_check_for_messages(wind_client_t* client);

int wind_create_window(wind_client_t* client, wind_window_t* window, struct wind_rect rect);
int wind_close_window(wind_client_t* client, wind_window_t* window);
int wind_invalidate(wind_client_t* client, wind_window_t* window);

int wind_set_window_title(wind_client_t* client, wind_window_t* window, const char* title);
int wind_set_titlebar_height(wind_client_t* client, wind_window_t* window, int height);

int wind_get_screen_dimensions(wind_client_t* client, struct wind_rect* rect);

int wind_set_special_window_attributes(wind_client_t* client, wind_window_t* window, enum wind_window_attributes attributes);

#endif